#include "PlatformPrecomp.h"
#include "CityCell.h"
#include "Component/GameLogicComponent.h"

CityCell::CityCell()
{
	m_overlayType = OVERLAY_NONE;
	Clear();
}

void CityCell::Clear()
{

	if (m_overlayType == OVERLAY_HEART)
	{
		GetCity()->ModLife(-1);
	}
	m_previewNum = 0;
	m_type = TYPE_BLANK;
	m_overlayType = OVERLAY_NONE;
	m_orientation = ORIENTATION_UP;
	m_graphicRotationDegrees = 0;

	for (int i=0; i < ORIENTATION_COUNT; i++)
	{
		m_socket[i] = SOCKET_NONE;
	}
	m_health = m_maxHealth = 1;
	m_baseColor = MAKE_RGBA(0,200,0,255);
}

void CityCell::RotateSockets90Degrees()
{
	eSocketType temp = m_socket[ORIENTATION_RIGHT];
	m_socket[ORIENTATION_RIGHT] = m_socket[ORIENTATION_UP];
	m_socket[ORIENTATION_UP] = m_socket[ORIENTATION_LEFT];
	m_socket[ORIENTATION_LEFT] = m_socket[ORIENTATION_DOWN];
	m_socket[ORIENTATION_DOWN] = temp;
}

void CityCell::Set( eCellType type, eCellOrientation orientation /*= ORIENTATION_UP*/, eOverlayType overlayType /*= OVERLAY_NONE*/ )
{
	Clear();
	m_type = type;
	m_orientation = orientation;
	m_overlayType = overlayType;
	m_bCanRotate = false;

	if (m_overlayType == OVERLAY_HEART)
	{
		GetCity()->ModLife(1);
	}
	//setup sockets

	switch (type)
	{
	case TYPE_4_BASIC:
		m_socket[ORIENTATION_LEFT] = SOCKET_PLUG;
		m_socket[ORIENTATION_RIGHT] = SOCKET_PLUG;
		m_socket[ORIENTATION_UP] = SOCKET_PLUG;
		m_socket[ORIENTATION_DOWN] = SOCKET_PLUG;
		m_bCanRotate = true;
		break;

	case TYPE_3_BASIC:
		m_socket[ORIENTATION_LEFT] = SOCKET_PLUG;
		m_socket[ORIENTATION_RIGHT] = SOCKET_PLUG;
		m_socket[ORIENTATION_UP] = SOCKET_PLUG;
		m_bCanRotate = true;
	break;

	case TYPE_2_BASIC:
		m_socket[ORIENTATION_UP] = SOCKET_PLUG;
		m_socket[ORIENTATION_DOWN] = SOCKET_PLUG;
		m_bCanRotate = true;
	break;

	case TYPE_1_GUN:
		m_baseColor = MAKE_RGBA(255,255,0,255);
		m_socket[ORIENTATION_UP] = SOCKET_WEAPON_STRAIGHT;
		m_socket[ORIENTATION_DOWN] = SOCKET_PLUG;
		break;

	case TYPE_3_ANGLED_RIGHT_GUN:
		m_baseColor = MAKE_RGBA(255,255,0,255);
		m_socket[ORIENTATION_UP] = SOCKET_WEAPON_DIAGONAL_RIGHT;
		m_socket[ORIENTATION_LEFT] = SOCKET_PLUG;
		m_socket[ORIENTATION_RIGHT] = SOCKET_PLUG;
		m_socket[ORIENTATION_DOWN] = SOCKET_PLUG;
		m_bCanRotate = true;
	break;
	
	case TYPE_3_GUN:
		m_baseColor = MAKE_RGBA(255,255,0,255);
		m_socket[ORIENTATION_UP] = SOCKET_WEAPON_STRAIGHT;
		m_socket[ORIENTATION_LEFT] = SOCKET_PLUG;
		m_socket[ORIENTATION_RIGHT] = SOCKET_PLUG;
		m_socket[ORIENTATION_DOWN] = SOCKET_PLUG;
		m_bCanRotate = true;
	break;

	case TYPE_2_GUN:
		m_baseColor = MAKE_RGBA(255,255,0,255);
		m_socket[ORIENTATION_UP] = SOCKET_WEAPON_STRAIGHT;
		m_socket[ORIENTATION_RIGHT] = SOCKET_PLUG;
		m_socket[ORIENTATION_LEFT] = SOCKET_PLUG;
		m_bCanRotate = true;
	break;

	case TYPE_2_CORNER_LEFT_GUN:
		m_baseColor = MAKE_RGBA(255,255,0,255);
		m_socket[ORIENTATION_UP] = SOCKET_WEAPON_DIAGONAL_LEFT;
		m_socket[ORIENTATION_RIGHT] = SOCKET_PLUG;
		m_socket[ORIENTATION_DOWN] = SOCKET_PLUG;
		m_bCanRotate = true;
	break;
	
	case TYPE_2_CORNER_BASIC:
		m_socket[ORIENTATION_UP] = SOCKET_PLUG;
		m_socket[ORIENTATION_RIGHT] = SOCKET_PLUG;
		m_bCanRotate = true;
	break;

	}


	switch (m_orientation)
	{
	case ORIENTATION_RIGHT:
		RotateSockets90Degrees();
		m_graphicRotationDegrees = 90;
		break;

	case ORIENTATION_DOWN:
		RotateSockets90Degrees();
		RotateSockets90Degrees();
		m_graphicRotationDegrees = 180;
		break;
	
	case ORIENTATION_LEFT:
		RotateSockets90Degrees();
		RotateSockets90Degrees();
		RotateSockets90Degrees();
		m_graphicRotationDegrees = 270;
		break;
	}
}


void CityCell::SetGridPos( int x, int y )
{
	m_posX = x;
	m_posY = y;
}

int CityCell::GetOpenSocketCount()
{
	int openSockets = 0;

	for (int i=0; i < ORIENTATION_COUNT; i++)
	{
		if (m_socket[i] == SOCKET_PLUG)
		{
			//but is something in the way?
			CityCell *pCell = GetCity()->GetNeighborCell(this, (eCellOrientation)i);

			if (pCell && pCell->m_type == TYPE_BLANK)
			{
				openSockets++;
			}
		}
	}

	return openSockets;
}

bool CityCell::IsClickable()
{
	if (GetGameLogic()->InPreviewMode())
	{
		if (m_previewNum == 0) return false; //we're not one of the new tiles
		return true;
	}

	for (int i=0; i < ORIENTATION_COUNT; i++)
	{
			if (m_socket[i] == SOCKET_WEAPON_DIAGONAL_LEFT ||
			m_socket[i] == SOCKET_WEAPON_DIAGONAL_RIGHT ||
			m_socket[i] == SOCKET_WEAPON_STRAIGHT)
			{
				return true;
			}
	}

	if (m_overlayType == OVERLAY_NUKE)
	{
		return true;
	}

	return false;
}

void CityCell::Rotate()
{
	GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_SOUND, "audio/rotate.wav", 1, TIMER_SYSTEM);
	RotateSockets90Degrees();
	m_graphicRotationDegrees += 90;

}

void CityCell::OnClicked()
{

	if (GetGameLogic()->InPreviewMode())
	{
		Rotate();
		return;
	}

	for (int i=0; i < ORIENTATION_COUNT; i++)
	{
		if (m_socket[i] == SOCKET_WEAPON_DIAGONAL_LEFT ||
			m_socket[i] == SOCKET_WEAPON_DIAGONAL_RIGHT ||
			m_socket[i] == SOCKET_WEAPON_STRAIGHT)
		{
			GetGameLogic()->FireBulletFromCell(this, i);
		}
		
	}

	if (m_overlayType == OVERLAY_NUKE)
	{
		GetGameLogic()->OnUseNuke();
		m_overlayType = OVERLAY_NONE;
	}

}

void CityCell::OnDamage( float damage )
{
	m_health -= damage;
	if (m_health <= 0)
	{
	
		GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_SOUND, "audio/city_die.wav", 1, TIMER_SYSTEM);

		m_health = 0;
		Clear();
	}
}

float CityCell::GetHealthPercent()
{
	return m_health / m_maxHealth;
}


uint32 CityCell::GetColor()
{
	if (GetGameLogic()->InPreviewMode() && m_previewNum > 0)
	{
		//special rules on coloring
		
		int flashSPeed = 1200;
		if (m_bCanRotate)
		{
			flashSPeed = 600;
		}
		return ColorCombine(MAKE_RGBA(255,255,255,255),m_baseColor,  0.6f+SinPulseByMS(flashSPeed)*.4f );

	}

	return ColorCombineMix(MAKE_RGBA(255,0,0,255),m_baseColor,  GetHealthPercent() );
}