#include "PlatformPrecomp.h"
#include "CityData.h"

CityData::CityData()
{
	m_sizeX = 0;
	m_sizeY = 0;
	m_life = 0;
}

CityData::~CityData()
{
}


void CityData::InitMap(int x, int y)
{
	m_sizeX = x;
	m_sizeY = y;
	m_cells.clear();
	m_cells.resize(x*y);
	m_life = 0;

	SetEvolveSeed(Random(RT_RAND_MAX));
	//setup odds of getting what

	//assign them their cell # so they can "look" around them properly
	for (int x=0; x < m_sizeX; x++)
	{
		for (int y=0; y < m_sizeY; y++)
		{
			GetCell(x,y)->SetGridPos(x,y);
		}
	}
	
}

void CityData::FillRandom()
{
	assert(m_sizeX != 0);
	for (int x=0; x < m_sizeX; x++)
	{
		for (int y=0; y < m_sizeY; y++)
		{
			GetCell(x,y)->Set( (eCellType) Random(7), ORIENTATION_UP, (eOverlayType) Random(2));
		}
	}
}

CityCell * CityData::GetCellSafe( int x, int y )
{
	if (x < 0 || y < 0 || x >= m_sizeX || y >= m_sizeY) return NULL;
	return GetCell(x,y);
}

CityCell * CityData::GetRandomCellOnRow(int row)
{
	return GetCell(m_rng.Random(m_sizeX), row);
}

void CityData::GenerateBase()
{
	//but some tries on the bottom

	CityCell *pCell=GetMiddleCell();
	pCell->Set(TYPE_4_BASIC, ORIENTATION_UP, OVERLAY_HEART);

	
	/*
	int tries = (int) ((float)GetSizeX()*0.6f);

	for (int i=0; i < tries; i++)
	{
	   CityCell *pCell=GetRandomCellOnRow(GetSizeY()-1);
	   pCell->Set(TYPE_3_BASIC, ORIENTATION_UP, OVERLAY_HEART);
	}
	*/
}

void CityData::GetListOfCellsWithOpenSockets(vector<CityCell*> *pList)
{
	for (int i=0; i < m_sizeX*m_sizeY; i++)
	{
		if (m_cells[i].GetOpenSocketCount() > 0)
		{
			pList->push_back(&m_cells[i]);
		}
	}
}

void CityData::GetListOfOpenCellsThatAreSocketable(vector<CityCell*> *pList)
{
	for (int i=0; i < m_sizeX*m_sizeY; i++)
	{
		if (m_cells[i].m_type == TYPE_BLANK)
		{
			CityCell *pNeighbor = GetNeighborWithSocketTouchingUs(&m_cells[i]);
			if (pNeighbor)
			{
				pList->push_back(&m_cells[i]);
			}
		}
	}
}

bool CityData::FillBlankWithNewSocket(CityCell *pCell)
{
	assert(pCell->m_type == TYPE_BLANK);
	
	//choose type
	eCellType type = (eCellType) m_weightRand.GetRandom(m_rng);
	
	eCellOrientation parentDir = GetDirectionOfNeighborWithSocketTouchingUs(pCell);

	eCellOrientation newOrientation = (eCellOrientation)m_rng.Random(ORIENTATION_COUNT);
	eOverlayType overlay = OVERLAY_NONE;

	//choose an overlay too?
	if (type == TYPE_2_BASIC || type == TYPE_3_BASIC || type == TYPE_4_BASIC || type == TYPE_2_CORNER_BASIC)
	{
		overlay = (eOverlayType) m_overlayWeightRand.GetRandom(m_rng);
		//LogMsg("overlay: %d", overlay);
	}
	
	pCell->Set(type, newOrientation, overlay);

	while(1)
	{
		if (pCell->m_socket[parentDir] == SOCKET_PLUG) break; //good to go

		//uh oh, rotate again
		newOrientation = (eCellOrientation) ((newOrientation+1)%ORIENTATION_COUNT);
		pCell->Set(type, newOrientation, overlay);
	}

	pCell->m_previewNum = m_curPreviewNum++;

	return true;
}

void CityData::AddToListUnique(CityCell *pCell, vector<CityCell*> *pCellList)
{
	//make sure it isn't already on the list

	for (uint32 i=0; i < pCellList->size(); i++)
	{
		if (pCellList->at(i) == pCell) return;
	}

	pCellList->push_back(pCell);
}

void CityData::AddBlankSocketableNeighborsAndCheckForDupes(CityCell *pCell, vector<CityCell*> *pCellList)
{
	for (int i=0; i < ORIENTATION_COUNT; i++)
	{
		if (pCell->m_socket[ (eCellOrientation)i] == SOCKET_PLUG)
		{
			CityCell *pTemp = GetNeighborCell(pCell, (eCellOrientation)i);
			if (pTemp)
			{
				if (pTemp->m_type == TYPE_BLANK)
				{
					AddToListUnique(pTemp, pCellList);
				}
			}

		}
	}
}

bool CityData::Evolve(int steps, eEvolveRules rules)
{
	//setup evolve probability

	switch(rules)
	{
	case EVOLVE_NORMAL:

		m_weightRand.Clear();
		m_weightRand.AddChoice(TYPE_2_CORNER_BASIC, 7*1000);
		m_weightRand.AddChoice(TYPE_4_BASIC, 2*1000);
		m_weightRand.AddChoice(TYPE_3_BASIC, 5*1000);
		m_weightRand.AddChoice(TYPE_2_BASIC, 7*1000);

		m_weightRand.AddChoice(TYPE_1_GUN, 2*1000);
		m_weightRand.AddChoice(TYPE_2_GUN, 2*1000);
		m_weightRand.AddChoice(TYPE_3_GUN, 3*1000);
		m_weightRand.AddChoice(TYPE_2_CORNER_LEFT_GUN, 1*1000);
		m_weightRand.AddChoice(TYPE_3_ANGLED_RIGHT_GUN, 0.5*1000);

		//setup overlays odds

		m_overlayWeightRand.Clear();
		m_overlayWeightRand.AddChoice(OVERLAY_NONE, 90);
		m_overlayWeightRand.AddChoice(OVERLAY_HEART, 3);
		m_overlayWeightRand.AddChoice(OVERLAY_GENERATOR, 8);
		m_overlayWeightRand.AddChoice(OVERLAY_NUKE, 5);
		break;

	case EVOLVE_RICH:

		m_weightRand.Clear();
		m_weightRand.AddChoice(TYPE_2_CORNER_BASIC, 7*1000);
		m_weightRand.AddChoice(TYPE_4_BASIC, 2*1000);
		m_weightRand.AddChoice(TYPE_3_BASIC, 5*1000);
		m_weightRand.AddChoice(TYPE_2_BASIC, 7*1000);

		m_weightRand.AddChoice(TYPE_1_GUN, 1*1000);
		m_weightRand.AddChoice(TYPE_2_GUN, 1*1000);
		m_weightRand.AddChoice(TYPE_3_GUN, 0.5*1000);
		m_weightRand.AddChoice(TYPE_2_CORNER_LEFT_GUN, 2*1000);
		m_weightRand.AddChoice(TYPE_3_ANGLED_RIGHT_GUN, 2*1000);

		//setup overlays odds

		m_overlayWeightRand.Clear();
		m_overlayWeightRand.AddChoice(OVERLAY_NONE, 20);
		m_overlayWeightRand.AddChoice(OVERLAY_HEART, 3);
		m_overlayWeightRand.AddChoice(OVERLAY_GENERATOR, 15);
		m_overlayWeightRand.AddChoice(OVERLAY_NUKE, 5);
		break;

	}


	//first, make a list of all blocks that have possible ways to evolve

	m_curPreviewNum = 1;
	vector<CityCell*> cellList;
	GetListOfOpenCellsThatAreSocketable(&cellList);

	//LogMsg("Have %d with valid open sockets", cellList.size());

	for (int i=0; i < steps; i++)
	{
		if (cellList.size() == 0)
		{
			LogMsg("Out of open sockets!");
			return false;
		}

		//choose one of the open sockets to work with randomly
		int listNum = m_rng.Random(cellList.size());
		CityCell *pCell = cellList[listNum];

		//ok, we'll put something new in here
		bool bSuccess = FillBlankWithNewSocket(pCell);
		
		if (bSuccess)
		{
			//Remove cell, it's no longer available
			cellList.erase(cellList.begin()+listNum);

			//however, by adding this new one, it may have added new opportunities
			AddBlankSocketableNeighborsAndCheckForDupes(pCell, &cellList);
		}
	}

	return true; //was able to evolve all of it
}

eCellOrientation CityData::GetOppositeOrientation(eCellOrientation orientation)
{
	switch (orientation)
	{
		case ORIENTATION_DOWN: return ORIENTATION_UP;
		case ORIENTATION_UP: return ORIENTATION_DOWN;
		case ORIENTATION_LEFT: return ORIENTATION_RIGHT;
		case ORIENTATION_RIGHT: return ORIENTATION_LEFT;
		default:

			assert(!"Error");;
	}
	return ORIENTATION_DOWN;
}

CityCell * CityData::GetNeighborWithSocketTouchingUs(CityCell *pCell)
{
	for (int i=0; i < ORIENTATION_COUNT; i++)
	{
		CityCell *pTemp = GetNeighborCell(pCell, (eCellOrientation)i);

		if (pTemp)
		{
			if (pTemp->m_socket[GetOppositeOrientation((eCellOrientation)i)] == SOCKET_PLUG)
			{
				//yeah, this'll work
				return pTemp;
			}
		}
	}
	return NULL;
}


eCellOrientation CityData::GetDirectionOfNeighborWithSocketTouchingUs(CityCell *pCell)
{
	for (int i=0; i < ORIENTATION_COUNT; i++)
	{
		CityCell *pTemp = GetNeighborCell(pCell, (eCellOrientation)i);

		if (pTemp)
		{
			if (pTemp->m_socket[GetOppositeOrientation((eCellOrientation)i)] == SOCKET_PLUG)
			{
				//yeah, this'll work
				return (eCellOrientation)i;
			}
		}
	}
	assert(!"Huh");
	return ORIENTATION_LEFT;
}

CityCell * CityData::GetNeighborCell(CityCell *pCell, eCellOrientation orientation)
{
	int x = pCell->m_posX;
	int y = pCell->m_posY;

	switch(orientation)
	{
	case ORIENTATION_LEFT:
		x--;
		break;
	case ORIENTATION_RIGHT:
		x++;
		break;
	case ORIENTATION_UP:
		y--;
		break;
	case ORIENTATION_DOWN:
		y++;
		break;
	default:

		assert(!"Error");;
	}

	return GetCellSafe(x,y);
}

void CityData::GetRectOfFilledArea( CL_Rect &r )
{
	r.bottom = m_sizeY/2;
	r.top = r.bottom;
	r.left = m_sizeX/2;
	r.right = r.left;

	//this assumes we start in the middle

	CityCell *pCell;
	for (int x=0; x < m_sizeX; x++)
		for (int y=0; y < m_sizeY; y++)
		{
			pCell = GetCell(x,y);
			if (pCell->m_type != TYPE_BLANK)
			{
				r.bottom = rt_max(y, r.bottom);
				r.top = rt_min(y, r.top);
				r.left = rt_min(x, r.left);
				r.right = rt_max(x, r.right);
			}
		}

}

void CityData::ClearPreviewInfo()
{
	for (int i=0; i < m_cells.size(); i++)
	{
		m_cells[i].m_previewNum = 0;
	}
}

void CityData::SetEvolveSeed( int seed )
{
	m_rng.SetRandomSeed(seed);
}

CityCell * CityData::GetMiddleCell()
{
	return GetCell(m_sizeX/2, m_sizeY/2);
}

vector<CityCell*> CityData::GetCellsWithOverlayType( eOverlayType overlay )
{
	vector<CityCell*> cells;

	for (int i=0; i < m_cells.size(); i++)
	{
		if (m_cells[i].m_overlayType == overlay)
		{
			cells.push_back(&m_cells[i]);
		} 
	}

	return cells;
}