//  ***************************************************************
//  CityData - Creation date: 08/25/2012
//  -------------------------------------------------------------
//  Robinson Technologies Copyright (C) 2012 - All Rights Reserved
//
//  ***************************************************************
//  Programmer(s):  Seth A. Robinson (seth@rtsoft.com)
//  ***************************************************************

#ifndef CityData_h__
#define CityData_h__

#include "CityCell.h"
#include "Math/WeightRand.h"

class CityData
{
public:
	CityData();
	virtual ~CityData();
	

	enum eEvolveRules
	{
		EVOLVE_NORMAL,
		EVOLVE_RICH
	};

	void InitMap(int x, int y);
	int GetSizeX() {return m_sizeX;}
	int GetSizeY() {return m_sizeY;}
	void FillRandom();

	CityCell * GetCell(int x, int y) {return &m_cells[y*GetSizeX()+x];}
	CityCell * GetCellSafe(int x, int y);

	void GenerateBase();
	CityCell * GetRandomCellOnRow(int row);
	bool Evolve(int steps, eEvolveRules rules);
	CityCell * GetNeighborCell(CityCell *pCell, eCellOrientation orientation);
	CityCell * GetNeighborWithSocketTouchingUs(CityCell *pCell);
	eCellOrientation GetOppositeOrientation(eCellOrientation orientation);
	
	void GetRectOfFilledArea(CL_Rect &r);
	void SetEvolveSeed(int seed);
	CityCell * GetMiddleCell();
	int GetLife() {return m_life;}
	void ModLife(int lifeMod) {m_life += lifeMod;}
	vector<CityCell*> GetCellsWithOverlayType(eOverlayType overlay);
	void ClearPreviewInfo();

protected:

	int m_curPreviewNum;
	int m_life;
	void GetListOfCellsWithOpenSockets(vector<CityCell*> *pList);
	void GetListOfOpenCellsThatAreSocketable(vector<CityCell*> *pList);
	bool FillBlankWithNewSocket(CityCell *pCell);
	void AddToListUnique(CityCell *pCell, vector<CityCell*> *pCellList);
	void AddBlankSocketableNeighborsAndCheckForDupes(CityCell *pCell, vector<CityCell*> *pCellList);
	eCellOrientation GetDirectionOfNeighborWithSocketTouchingUs(CityCell *pCell);
	int m_sizeX,m_sizeY;
	vector<CityCell> m_cells;
	CWeightRand m_weightRand;
	CRandom m_rng;
	CWeightRand m_overlayWeightRand;

private:
};

#endif // CityData_h__