#include "PlatformPrecomp.h"
#include "CityRenderComponent.h"
#include "Entity/EntityUtils.h"
#include "CityData.h"
#include "GameLogicComponent.h"

CityRenderComponent::CityRenderComponent()
{
	SetName("CityRender");
}

CityRenderComponent::~CityRenderComponent()
{

}

void CityRenderComponent::OnAdd(Entity *pEnt)
{
	EntityComponent::OnAdd(pEnt);
	m_vCellSize = CL_Vec2f(64,64);

	m_pPos2d = &GetParent()->GetVar("pos2d")->GetVector2();
	//m_pSize2d = &GetParent()->GetVar("size2d")->GetVector2();
	
	m_pScale2d = &GetParent()->GetVarWithDefault("scale2d", Variant(1.0f, 1.0f))->GetVector2();
	//m_pRotation = &GetParent()->GetVar("rotation")->GetFloat();  //in degrees

	m_pColor = &GetParent()->GetVarWithDefault("color", Variant(MAKE_RGBA(255,255,255,255)))->GetUINT32();
	m_pColorMod = &GetParent()->GetVarWithDefault("colorMod", Variant(MAKE_RGBA(255,255,255,255)))->GetUINT32();
	m_pAlpha = &GetParent()->GetVarWithDefault("alpha", Variant(1.0f))->GetFloat();
	//m_pAlignment = &GetParent()->GetVar("alignment")->GetUINT32();
	
	//register ourselves to render if the parent does
	GetParent()->GetFunction("OnRender")->sig_function.connect(1, boost::bind(&CityRenderComponent::OnRender, this, _1));
	GetParent()->GetFunction("OnUpdate")->sig_function.connect(1, boost::bind(&CityRenderComponent::OnUpdate, this, _1));
	GetParent()->GetFunction("OnInput")->sig_function.connect(1, boost::bind(&CityRenderComponent::OnInput, this, _1));

	m_spriteGrid.LoadFile("game/sprite_grid.rttex");
	m_spriteGrid.SetupAnim(16,8);
	m_surfBullet.LoadFile("game/bullet.rttex");

	m_surfRock.LoadFile("game/rock.rttex");
	m_surfRock.SetupAnim(8, 1);
	//*m_pPos2d = CL_Vec2f(100,-200);
	*m_pScale2d = CL_Vec2f(0.5f, 0.5f);
}

void CityRenderComponent::OnRemove()
{
	EntityComponent::OnRemove();
}

void CityRenderComponent::RenderCell(float x, float y, CityCell *pCell)
{
	//DrawFilledRect(CL_Vec2f(x,y), m_vCellSize, MAKE_RGBA(Random(255),255,255,255));
	//uint32 color = GetBrightColor();

	uint32 color = pCell->GetColor();
	
	CL_Vec2f vRotationPt = CL_Vec2f(x+(32.0f*m_pScale2d->x) ,y+ (32.0f*m_pScale2d->y));
	
	switch (pCell->m_type)
	{
	case TYPE_4_BASIC:
		m_spriteGrid.BlitScaledAnim(x,y, 1, 1, *m_pScale2d, ALIGNMENT_UPPER_LEFT, color, pCell->m_graphicRotationDegrees, vRotationPt);
		break;

	case TYPE_3_ANGLED_RIGHT_GUN:
		m_spriteGrid.BlitScaledAnim(x,y, 7, 1, *m_pScale2d, ALIGNMENT_UPPER_LEFT, color, pCell->m_graphicRotationDegrees, vRotationPt);
	break;

	case TYPE_3_GUN:
		m_spriteGrid.BlitScaledAnim(x,y, 5, 1, *m_pScale2d, ALIGNMENT_UPPER_LEFT, color, pCell->m_graphicRotationDegrees, vRotationPt);
	break;

	case TYPE_1_GUN:
		m_spriteGrid.BlitScaledAnim(x,y, 3, 1, *m_pScale2d, ALIGNMENT_UPPER_LEFT, color, pCell->m_graphicRotationDegrees, vRotationPt);
	break;

	case TYPE_3_BASIC:
		m_spriteGrid.BlitScaledAnim(x,y, 9, 1, *m_pScale2d, ALIGNMENT_UPPER_LEFT, color, pCell->m_graphicRotationDegrees, vRotationPt);
		break;

	case TYPE_2_BASIC:
		m_spriteGrid.BlitScaledAnim(x,y, 11, 1, *m_pScale2d, ALIGNMENT_UPPER_LEFT, color, pCell->m_graphicRotationDegrees, vRotationPt);
		break;

	case TYPE_2_CORNER_BASIC:
		m_spriteGrid.BlitScaledAnim(x,y, 2, 2, *m_pScale2d, ALIGNMENT_UPPER_LEFT, color, pCell->m_graphicRotationDegrees, vRotationPt);
		break;

	case TYPE_2_CORNER_LEFT_GUN:
		m_spriteGrid.BlitScaledAnim(x,y, 4, 2, *m_pScale2d, ALIGNMENT_UPPER_LEFT, color, pCell->m_graphicRotationDegrees, vRotationPt);
		break;

	case TYPE_2_GUN:
		m_spriteGrid.BlitScaledAnim(x,y, 6, 2, *m_pScale2d, ALIGNMENT_UPPER_LEFT, color, pCell->m_graphicRotationDegrees, vRotationPt);
		break;

	}

	switch (pCell->m_overlayType)
	{
	
	case OVERLAY_HEART:
		m_spriteGrid.BlitScaledAnim(x,y, 1, 3, *m_pScale2d, ALIGNMENT_UPPER_LEFT);
		break;

	case OVERLAY_NUKE:
		m_spriteGrid.BlitScaledAnim(x,y, 3, 3, *m_pScale2d, ALIGNMENT_UPPER_LEFT);
		break;

	case OVERLAY_GENERATOR:
		m_spriteGrid.BlitScaledAnim(x,y, 5, 3, *m_pScale2d, ALIGNMENT_UPPER_LEFT);
		break;

	}

}

void CityRenderComponent::RenderMap(CL_Vec2f vPos )
{
	for (int x = 0; x < m_pCityData->GetSizeX(); x++)
	{
		for (int y = 0; y < m_pCityData->GetSizeY(); y++)
		{
			RenderCell(vPos.x + (x*m_vCellSize.x* m_pScale2d->x), vPos.y + (y*m_vCellSize.y*m_pScale2d->y), m_pCityData->GetCell(x,y));
		}
	}
}

void CityRenderComponent::RenderBullets(CL_Vec2f vPos)
{
	list<Bullet>::iterator itor = m_bullets.begin();

	for (;itor != m_bullets.end(); itor++)
	{
		m_surfBullet.BlitScaledAnim(vPos.x+ (itor->m_vPos.x*m_pScale2d->x), vPos.y+ (itor->m_vPos.y*m_pScale2d->y), 0,0, *m_pScale2d*itor->m_scale, ALIGNMENT_CENTER, MAKE_RGBA(255,255,0,itor->m_alpha*255));
	}
}



void CityRenderComponent::OnRender(VariantList *pVList)
{
	CL_Vec2f vFinalPos = pVList->m_variant[0].GetVector2()+*m_pPos2d;
	RenderMap(vFinalPos);
	RenderBullets(vFinalPos);
	RenderRocks(vFinalPos);
	RenderEffects(vFinalPos);
}

#define C_BULLET_TIMEOUT_MS 5000

void CityRenderComponent::UpdateBullets()
{
	list<Bullet>::iterator itor = m_bullets.begin();

	for (;itor != m_bullets.end();)
	{

		if (itor->m_startTime + C_BULLET_TIMEOUT_MS < GetTick(TIMER_GAME))
		{
			itor->m_bDeleteASAP = true;	
			
		}

		if (itor->m_bShrinkAndDie)
		{
			itor->m_scale -= 0.1f*GetBaseApp()->GetGameDelta();
			itor->m_alpha -= 0.1f*GetBaseApp()->GetGameDelta();
			if (itor->m_scale <= 0.1f || itor->m_alpha <= 0.1f)
			{
				itor->m_bDeleteASAP = true;
			}
		} else
		{

		

		//update bullets travel
			float speed = 5.0f;
			float damage = 0.8f;

			itor->m_vPos += itor->m_vDir*speed*GetBaseApp()->GetDelta();
			
			//check for collisions
			CityCell *pCell = GetCellByWorldPos(itor->m_vPos);

			if (pCell && pCell != itor->m_pParentCell && pCell->m_type != TYPE_BLANK)
			{
				//we can attack it!  But wait, are we fully inside its square?
				CL_Vec2f vCellCenter = GetCenterCellPosWorld(pCell);

				CL_Vec2f vDiff = vCellCenter-itor->m_vPos;
				if (vDiff.length() <  (m_vCellSize.x/2+ ( m_surfBullet.GetFrameSize().x/2)))
				{
					itor->m_bShrinkAndDie = true;
					pCell->OnDamage(0.5f);
					
				}
			}

			//collision against rocks
			list<Rock>::iterator rockItor = m_rocks.begin();

			for (;rockItor != m_rocks.end(); rockItor++)
			{
				if (rockItor->m_bDeleteASAP) continue;

				CL_Vec2f vDiff = rockItor->m_vPos-itor->m_vPos;
				if (vDiff.length() <  ( m_surfBullet.GetFrameSize().x/2+( (m_surfRock.GetFrameSize().x*rockItor->m_scale)* 0.5f))  )
				{
					itor->m_bShrinkAndDie = true;	
					rockItor->OnDamage(damage, true);
				}

			}
		}

			//delete it?
			if (itor->m_bDeleteASAP)
			{
				list<Bullet>::iterator tempItor = itor;
				itor++;
				m_bullets.erase(tempItor);
				continue;
			}

			itor++;

	}
}

void CityRenderComponent::UpdateCamera(bool bInstant)
{
	CL_Rect r;

	GetCity()->GetRectOfFilledArea(r);

	//LogMsg("Rect %d, %d, and %d, %d", r.top, r.left, r.right, r.bottom);

	int blocksToFitOnScreen = rt_max(r.get_width(), r.get_height());
	blocksToFitOnScreen += 6; //just some extra spacing
	blocksToFitOnScreen = rt_max(blocksToFitOnScreen, 10);


	float scaleX = GetScreenSizeXf() / (blocksToFitOnScreen* m_vCellSize.x);
	float scaleY = GetScreenSizeYf() / (blocksToFitOnScreen* m_vCellSize.y);

	float finalScale = rt_min(scaleX, scaleY);
	//align to 0,0
	
	CL_Vec2f vPos;
	vPos.x = -(GetCity()->GetSizeX()/2) * (m_vCellSize.x*finalScale);
	vPos.y = -(GetCity()->GetSizeY()/2) * (m_vCellSize.y*finalScale);

	//now pull back
	float blocksFittableX  = (GetScreenSizeXf() / (m_vCellSize.x* finalScale)); 
	vPos.x +=  ((blocksFittableX/2)-0.5f) * (m_vCellSize.x* finalScale);

	float blocksFittableY  = (GetScreenSizeYf() / (m_vCellSize.y* finalScale)); 
	vPos.y +=  ((blocksFittableY/2)-0.5f) * (m_vCellSize.y* finalScale);
	
	int timeMS = 400;
	if (bInstant) timeMS = 0;

	//actually change the scale
	MorphToVec2Entity(GetParent(), "scale2d", CL_Vec2f(finalScale,finalScale), timeMS);
	//*m_pScale2d = CL_Vec2f(finalScale,finalScale);

	//and pos
	MorphToVec2Entity(GetParent(), "pos2d", vPos, timeMS);

}


void CityRenderComponent::OnUpdate(VariantList *pVList)
{
	UpdateBullets();
	UpdateRocks();
	UpdateEffects();

}

void CityRenderComponent::SetCityData( CityData* pData )
{
	m_pCityData = pData;
}

CityCell * CityRenderComponent::GetCellByScreenPos(CL_Vec2f vPos)
{
	vPos -= *m_pPos2d; //remove any drawing offset

	//divide to get the correct cell
	int x = (int)(vPos.x / (m_vCellSize.x*m_pScale2d->x) ) ;
	int y = (int) (vPos.y / (m_vCellSize.y*m_pScale2d->y) ) ;
	return m_pCityData->GetCellSafe(x,y);
}

CityCell * CityRenderComponent::GetCellByWorldPos(CL_Vec2f vPos)
{
	//vPos -= *m_pPos2d; //remove any drawing offset

	//divide to get the correct cell
	int x = (int)(vPos.x / (m_vCellSize.x) ) ;
	int y = (int) (vPos.y / (m_vCellSize.y) ) ;
	return m_pCityData->GetCellSafe(x,y);
}


void CityRenderComponent::OnInput( VariantList *pVList )
{
	//0 = message type, 1 = parent coordinate offset
	CL_Vec2f pt = pVList->Get(1).GetVector2();

	uint32 fingerID = 0;
	if (pVList->Get(2).GetType() == Variant::TYPE_UINT32)
	{
		fingerID = pVList->Get(2).GetUINT32();
	}

	switch (eMessageType( int(pVList->Get(0).GetFloat())))
	{
	case MESSAGE_TYPE_GUI_CLICK_START:
		{
			CityCell *pCell = GetCellByScreenPos(pt);
			if (pCell)
			{
				GetGameLogic()->OnClickCell(pCell);
			} else
			{
				LogMsg("Didn't click anything");
			}
		}
		//HandleClickStart(pt, fingerID);
		break;
	case MESSAGE_TYPE_GUI_CLICK_END:
		//HandleClickEnd(pt, fingerID);
		break;
	case MESSAGE_TYPE_GUI_CLICK_MOVE:
		//HandleClickMove(pt, fingerID);
		break;
	default:;
	}	

}

CL_Vec2f CityRenderComponent::GetCenterCellPosWorld( CityCell *pCell )
{
	CL_Vec2f vPos = CL_Vec2f(pCell->m_posX*m_vCellSize.x + m_vCellSize.x/2, pCell->m_posY*m_vCellSize.y+ m_vCellSize.y/2);
	return vPos;
}

Bullet * CityRenderComponent::AddBullet( CL_Vec2f vPos, CityCell *pParentCell, CL_Vec2f vDir )
{

	GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_SOUND, "audio/shoot.wav", 1, TIMER_SYSTEM);

	Bullet b;
	b.m_pParentCell = pParentCell;
	b.m_vDir = vDir;
	b.m_vPos = vPos;
	b.m_startTime = GetTick(TIMER_GAME);
	m_bullets.push_back(b);

	return &m_bullets.back();
}

const int C_ROCK_TIMEOUT_MS = 12000;

void CityRenderComponent::RenderRocks(CL_Vec2f vPos)
{
	list<Rock>::iterator itor = m_rocks.begin();

	for (;itor != m_rocks.end(); itor++)
	{
		float x = vPos.x+ (itor->m_vPos.x*m_pScale2d->x);
		float y = vPos.y+ (itor->m_vPos.y*m_pScale2d->y);

		m_surfRock.BlitScaledAnim(x, y, 0,0, *m_pScale2d*itor->m_scale, ALIGNMENT_CENTER, itor->GetColor(),
			itor->m_radRotation, CL_Vec2f(x,y));
	}
}

void CityRenderComponent::UpdateRocks()
{
	list<Rock>::iterator itor = m_rocks.begin();

	float worldCellSize = m_vCellSize.x * m_pScale2d->x;

	for (;itor != m_rocks.end();)
	{
		if (itor->m_startTime + C_ROCK_TIMEOUT_MS < GetTick(TIMER_GAME))
		{
			itor->m_bDeleteASAP = true;	
		}

		//update rocks travel
		
		itor->m_vPos += itor->m_vDir*itor->m_speed*GetBaseApp()->GetDelta();
		itor->m_radRotation += itor->m_rotationSpeed*GetBaseApp()->GetDelta();
	
		//check for collisions on space station or whatever it is

		const int maxCells = 5;

		CityCell *pCellsToCheck[maxCells];
		for (int i=0; i <maxCells; i++)
		{
			pCellsToCheck[i] = NULL;
		}

		int curCell = 0;

		pCellsToCheck[curCell++] = GetCellByWorldPos(itor->m_vPos);

		if (itor->m_scale > 0.6f)
		{
			//it's a biggy, check surrounding cells too
			pCellsToCheck[curCell++] = GetCellByWorldPos(itor->m_vPos+CL_Vec2f(0,worldCellSize));
			pCellsToCheck[curCell++] = GetCellByWorldPos(itor->m_vPos+CL_Vec2f(0,-worldCellSize));
			pCellsToCheck[curCell++] = GetCellByWorldPos(itor->m_vPos+CL_Vec2f(worldCellSize,worldCellSize));
			pCellsToCheck[curCell++] = GetCellByWorldPos(itor->m_vPos+CL_Vec2f(-worldCellSize,-worldCellSize));
		}

		for (int i=0; i <maxCells; i++)
		{
			if (!pCellsToCheck[i]) continue;

			CityCell *pCell = pCellsToCheck[i];

			if (pCell->m_type != TYPE_BLANK)
			{
				//we can attack it!  But wait, are we fully inside its square?
				CL_Vec2f vCellCenter = GetCenterCellPosWorld(pCell);

				CL_Vec2f vDiff = vCellCenter-itor->m_vPos;
				if (vDiff.length() <  ( m_vCellSize.x/2+( (m_surfRock.GetFrameSize().x*itor->m_scale)* 0.5f))  )
				{
					float damage = itor->m_scale;

					pCell->OnDamage(damage);
					itor->OnDamage(damage, false);
					if (itor->m_bDeleteASAP)
					{
						break;
					}
				}
			}
		}

		//delete it?
		if (itor->m_bDeleteASAP)
		{
			list<Rock>::iterator tempItor = itor;
			itor++;
			m_rocks.erase(tempItor);
			continue;
		}

		itor++;
	}
}

Rock * CityRenderComponent::AddRock( float scale, float speed )
{
	Rock r;
	
	float radDir = Random(M_PI*2.0f);
	float dist = 640;
	r.m_speed = speed;
	r.m_baseColor = MAKE_RGBA(50,50,255,255);
	r.m_vPos = GetCenterCellPosWorld(GetCity()->GetMiddleCell()) +  CL_Vec2f(sin(radDir), cos(radDir))*dist;
	r.m_radRotation =  Random(M_PI*2.0f);
	r.m_rotationSpeed = RandomRangeFloat(-2, 2);
	r.m_scale = scale;
	r.m_health = r.m_maxHealth = scale*3;
	//point at base
	CL_Vec2f vTarget = GetCenterCellPosWorld(GetCity()->GetMiddleCell());

	//add some randomness
	vTarget.x += RandomRangeFloat(-200, 200);
	vTarget.y += RandomRangeFloat(-200, 200);

	r.m_vDir = vTarget-r.m_vPos;
	r.m_vDir.normalize();

	r.m_startTime = GetTick(TIMER_GAME);
	m_rocks.push_back(r);
	return &m_rocks.back();
}


void CityRenderComponent::RenderEffects(CL_Vec2f vPos)
{
	list<Effect>::iterator itor = m_effects.begin();

	for (;itor != m_effects.end(); itor++)
	{
		itor->m_pSurfAnim->BlitScaledAnim(vPos.x+ (itor->m_vPos.x*m_pScale2d->x), vPos.y+ (itor->m_vPos.y*m_pScale2d->y), itor->m_curFrame,0, *m_pScale2d*(itor->m_scale), ALIGNMENT_CENTER, itor->m_color);
	}
}

void CityRenderComponent::UpdateEffects()
{
	list<Effect>::iterator itor = m_effects.begin();

	float worldCellSize = m_vCellSize.x * m_pScale2d->x;

	for (;itor != m_effects.end();)
	{
		itor->Update();
		//delete it?
		if (itor->m_bDeleteASAP)
		{
			list<Effect>::iterator tempItor = itor;
			itor++;
			m_effects.erase(tempItor);
			continue;
		}

		itor++;
	}
}


void CityRenderComponent::AddEffect( Effect::eType type, float scale, CL_Vec2f vWorldPos, uint32 color )
{
	Effect e;
	e.m_type = type;
	e.m_color = color;
	e.m_vPos = vWorldPos;
	e.m_curFrame = 0;
	e.m_timeBetweenFramesMS = 50;
	e.m_lastFrame = 7;
	e.m_pSurfAnim = &GetGameLogic()->GetRenderComponent()->m_surfRock;
	e.SetAnimTimer();
	e.m_bDeleteASAP = false;
	e.m_scale = scale;
	m_effects.push_back(e);
	
}

CL_Vec2f CityRenderComponent::WorldToScreen( CL_Vec2f vWorld )
{
	return CL_Vec2f( m_pPos2d->x+ (vWorld.x*m_pScale2d->x),
		m_pPos2d->y+ (vWorld.y*m_pScale2d->y));
}

CL_Vec2f CityRenderComponent::ScreenToWorld( CL_Vec2f vScreen)
{
	vScreen -= *m_pPos2d; //remove any drawing offset

	//divide to get the correct cell
	float x = (vScreen.x / (m_vCellSize.x*m_pScale2d->x) ) ;
	float y =  (vScreen.y / (m_vCellSize.y*m_pScale2d->y) ) ;

	return CL_Vec2f(x*m_vCellSize.x, y*m_vCellSize.y);
}

void CityRenderComponent::DestroyAllRocks(bool bCollectResources)
{
	list<Rock>::iterator itor = m_rocks.begin();

	for (;itor != m_rocks.end();)
	{
		itor->OnDamage(100, bCollectResources);
		itor++;
	}
}


void Rock::OnDamage( float damage, bool bAddResources )
{
	if (m_bDeleteASAP || m_health <= 0) return;

	m_health -= damage;
	if (m_health <= 0)
	{
		m_bDeleteASAP = true;
		GetGameLogic()->GetRenderComponent()->AddEffect(Effect::EFFECT_EXPLOSION, m_scale, m_vPos, GetColor());

		if (damage < 5)
		{
			//it's not a nuke, play sfx
			GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_SOUND, "audio/rock_break.wav", 1, TIMER_SYSTEM);

		}
		if (bAddResources)
		{
			int resource = 1;
			if (m_scale >= 0.6f)
			{
				resource++;
			}
			GetGameLogic()->AddResourcesFromPos(GetGameLogic()->GetRenderComponent()->WorldToScreen(m_vPos), resource);
		}
	} else
	{
		//still alive, but play damage noise
		GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_SOUND, "audio/rock_break_small.wav", 1, TIMER_SYSTEM);

	}
}

void Effect::SetAnimTimer()
{
	m_frameTimer = GetTick(TIMER_GAME)+m_timeBetweenFramesMS;
}

void Effect::Update()
{
	if (m_frameTimer < GetTick(TIMER_GAME))
	{
		m_curFrame++;
		if (m_curFrame > m_lastFrame)
		{
			m_curFrame--;
			m_bDeleteASAP = true;
		} else
		{
			SetAnimTimer();
		}
	}
}