//  ***************************************************************
//  CityRenderComponent - Creation date: 8/25/2012 Tokyo Time
//  -------------------------------------------------------------
//  Robinson Technologies Copyright (C) 2012 - All Rights Reserved
//
//  ***************************************************************
//  Programmer(s):  Seth A. Robinson (seth@rtsoft.com)
//  ***************************************************************

#ifndef CityRenderComponent_h__
#define CityRenderComponent_h__

#include "Entity/Component.h"
#include "Entity/Entity.h"
#include "Renderer/SurfaceAnim.h"
#include "CityRenderComponent.h"

class CityCell;

class Bullet
{
public:
	Bullet()
	{
		m_bDeleteASAP = false;
		m_bShrinkAndDie = false;
		m_scale = 1.0f;
		m_alpha = 1.0f;
	}
	CL_Vec2f m_vPos;
	CL_Vec2f m_vDir;
	CityCell *m_pParentCell;
	unsigned int m_startTime;
	bool m_bDeleteASAP;
	bool m_bShrinkAndDie;
	float m_scale;
	float m_alpha;
};


class Rock
{
public:
	Rock()
	{
		m_bDeleteASAP = false;
	}

	uint32 GetColor()
	{
		return ColorCombineMix(MAKE_RGBA(255,0,0,255),m_baseColor,  GetHealthPercent() );
	}

	float GetHealthPercent()
	{
		return m_health / m_maxHealth;
	}

	void OnDamage(float damage, bool bAddResources);

	CL_Vec2f m_vPos;
	CL_Vec2f m_vDir;
	unsigned int m_startTime;
	bool m_bDeleteASAP;
	float m_speed;
	float m_radRotation, m_rotationSpeed;
	float m_scale;
	float m_maxHealth, m_health;
	uint32 m_baseColor;
};

class Effect
{
public:
	enum eType
	{
		EFFECT_EXPLOSION
	};

	void SetAnimTimer();
	void Update();

	CL_Vec2f m_vPos;
	float m_scale;
	SurfaceAnim *m_pSurfAnim;
	int m_curFrame;
	int m_lastFrame;
	int m_timeBetweenFramesMS;
	uint32 m_color;
	uint32 m_frameTimer;
	bool m_bDeleteASAP;
	eType m_type;

};

class CityCell;
class CityData;

class CityRenderComponent: public EntityComponent
{
public:
	CityRenderComponent();
	virtual ~CityRenderComponent();

	virtual void OnAdd(Entity *pEnt);
	virtual void OnRemove();
	void SetCityData(CityData* pData);
	Bullet * AddBullet(CL_Vec2f vPos, CityCell *pParentCell, CL_Vec2f vDir);
	CityCell * GetCellByScreenPos(CL_Vec2f vPos);
	CL_Vec2f GetCenterCellPosWorld( CityCell *pCell );
	void UpdateCamera(bool bInstant = false);
	Rock * AddRock(float size, float speed);
	void AddEffect(Effect::eType type, float scale, CL_Vec2f vWorldPos, uint32 color);
	CL_Vec2f WorldToScreen(CL_Vec2f vWorld);
	CL_Vec2f ScreenToWorld( CL_Vec2f vScreen);
	void DestroyAllRocks(bool bCollectResources = true);
	SurfaceAnim m_surfBullet;
	SurfaceAnim m_surfRock;
	/*
	uint32 *m_pAlignment;
	float *m_pRotation; //in degrees
	*/

	SurfaceAnim m_spriteGrid;

private:

	void OnRender(VariantList *pVList);
	void OnUpdate(VariantList *pVList);
	void RenderMap(CL_Vec2f vPos );
	void RenderCell(float x, float y, CityCell *pCell);
	void OnInput( VariantList *pVList );
	void UpdateBullets();
	void RenderBullets(CL_Vec2f vPos);
	CityCell * GetCellByWorldPos(CL_Vec2f vPos);
	void UpdateRocks();
	void RenderRocks(CL_Vec2f vPos);
	void UpdateEffects();
	void RenderEffects(CL_Vec2f vPos);
	CityData *m_pCityData;
	CL_Vec2f *m_pPos2d;
	
	//CL_Vec2f *m_pSize2d;
	CL_Vec2f *m_pScale2d;
	uint32 *m_pColor;
	uint32 *m_pColorMod;
	float *m_pAlpha;

	CL_Vec2f m_vCellSize;

	list<Bullet> m_bullets;
	list<Rock> m_rocks;
	list<Effect> m_effects;


};

#endif // CityRenderComponent_h__