#include "PlatformPrecomp.h"
#include "GameLogicComponent.h"
#include "Entity/EntityUtils.h"
#include "CityRenderComponent.h"
#include "GUI/MainMenu.h"
#include "GUI/GameGUI.h"
#include "GUI/UpgradeMenu.h"

GameLogicComponent *g_pGameLogic = NULL;

GameLogicComponent * GetGameLogic()
{
	return g_pGameLogic;
}

CityData * GetCity()
{
	return g_pGameLogic->GetCity();
}

GameLogicComponent::GameLogicComponent()
{
	SetName("GameLogic");
	m_pCityRender = NULL;
	g_pGameLogic = this;
	Reset();
}

GameLogicComponent::~GameLogicComponent()
{
	GetParent()->RemoveComponentByName("CityRender");
}

void GameLogicComponent::OnAdd(Entity *pEnt)
{
	EntityComponent::OnAdd(pEnt);

	m_pPos2d = &GetParent()->GetVar("pos2d")->GetVector2();
	
	/*
	m_pSize2d = &GetParent()->GetVar("size2d")->GetVector2();
	m_pScale2d = &GetParent()->GetVarWithDefault("scale2d", Variant(1.0f, 1.0f))->GetVector2();
	m_pRotation = &GetParent()->GetVar("rotation")->GetFloat();  //in degrees

	m_pColor = &GetParent()->GetVarWithDefault("color", Variant(MAKE_RGBA(255,255,255,255)))->GetUINT32();
	m_pColorMod = &GetParent()->GetVarWithDefault("colorMod", Variant(MAKE_RGBA(255,255,255,255)))->GetUINT32();
	m_pAlpha = &GetParent()->GetVarWithDefault("alpha", Variant(1.0f))->GetFloat();
	m_pAlignment = &GetParent()->GetVar("alignment")->GetUINT32();
	*/

	//register ourselves to render if the parent does
	GetParent()->GetFunction("OnRender")->sig_function.connect(1, boost::bind(&GameLogicComponent::OnRender, this, _1));
	GetParent()->GetFunction("OnUpdate")->sig_function.connect(1, boost::bind(&GameLogicComponent::OnUpdate, this, _1));
	
	//component functions
	GetFunction("InitMap")->sig_function.connect(1, boost::bind(&GameLogicComponent::InitMap, this, _1));
	GetFunction("InitDemo")->sig_function.connect(1, boost::bind(&GameLogicComponent::InitDemo, this, _1));
	GetFunction("OnQuit")->sig_function.connect(1, boost::bind(&GameLogicComponent::OnQuit, this, _1));
	GetFunction("EndRockMode")->sig_function.connect(1, boost::bind(&GameLogicComponent::OnEndRockMode, this, _1));
	GetFunction("UpdateResourceDisplay")->sig_function.connect(1, boost::bind(&GameLogicComponent::OnUpdateResourceDisplay, this, _1));
	GetFunction("OnSay")->sig_function.connect(1, boost::bind(&GameLogicComponent::OnSay, this, _1));
	GetFunction("OnInitUpgradeMenu")->sig_function.connect(1, boost::bind(&GameLogicComponent::OnInitUpgradeMenu, this, _1));
	GetFunction("AddResourceFromPos")->sig_function.connect(1, boost::bind(&GameLogicComponent::AddResourceFromPos, this, _1));
	GetFunction("ShowProgressBar")->sig_function.connect(1, boost::bind(&GameLogicComponent::ShowProgressBar, this, _1));


}

void GameLogicComponent::OnRemove()
{
	EntityComponent::OnRemove();
}

void GameLogicComponent::OnUseNuke()
{
	LogMsg("using nuke!");
	
	for (int i=0; i < 20; i++)
	{

		GetGameLogic()->GetRenderComponent()->AddEffect(Effect::EFFECT_EXPLOSION, RandomRangeFloat(0.3, 2),
			m_pCityRender->ScreenToWorld(CL_Vec2f(RandomRange(40, GetScreenSizeXf()-80),RandomRange(40, GetScreenSizeYf()-80))),
			MAKE_RGBA(255,255,255,RandomRange(40,255))
			);
	}

	m_pCityRender->DestroyAllRocks();
	GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_SOUND, "audio/nuke.wav", 1, TIMER_SYSTEM);

}

void GameLogicComponent::OnRender(VariantList *pVList)
{
	CL_Vec2f vFinalPos = pVList->m_variant[0].GetVector2()+*m_pPos2d;
}

void GameLogicComponent::OnUpdateResourceDisplay(VariantList *pVList)
{
	SetResourceText(m_resources);
}

void GameLogicComponent::AddResourcesFromPos(CL_Vec2f vScreenPos, int resources)
{

	GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_SOUND, "audio/resource2.wav", 1, TIMER_SYSTEM);

	//create a text thing and make it float to where the resource GUI is
	Entity *pEnt = CreateTextLabelEntity(GetEntityRoot()->GetEntityByName("GameGUI"), "Misc", vScreenPos.x, vScreenPos.y, "`2"+toString(resources)+"``");
	SetAlignmentEntity(pEnt, ALIGNMENT_CENTER);
	SetupTextEntity(pEnt, FONT_LARGE);
	ZoomToPositionEntity(pEnt,  CL_Vec2f(GetScreenSizeXf() - iPadMapX(50),5), 1000);
	FadeOutAndKillEntity(pEnt, false, 300, 700);

	//and actually give them it
	m_resources += resources;
	GetMessageManager()->CallComponentFunction(this, 950, "UpdateResourceDisplay");
}

void GameLogicComponent::OnSay(VariantList *pVList)
{
	//TODO add sfx for Onsay
	string msg = pVList->Get(0).GetString();
	int timeToShow = pVList->Get(1).GetUINT32();

	Entity *pGameGUI = GetEntityRoot()->GetEntityByName("GameGUI");

	//remove any old messages hanging around
	GetEntityRoot()->RemoveEntityByName("MiscMessage", true); 
	
	if (m_level != 10)
	{
		GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_SOUND, "audio/resource.wav", 1, TIMER_SYSTEM);
	}

	Entity *pEnt = CreateTextBoxEntity(pGameGUI, "MiscMessage", CL_Vec2f(20, 50), CL_Vec2f(600, 200), msg);
	SetupTextEntity(pEnt, FONT_LARGE, 0.7f);
	TypeTextLabelEntity(pEnt, 0, 30);
	FadeOutAndKillEntity(pEnt, true, 500, timeToShow-500);
}

void GameLogicComponent::Say(string msg, int delayMS, int timeToShowMS)
{
	VariantList vList(msg, (uint32)timeToShowMS);
	GetMessageManager()->CallComponentFunction(this, delayMS, "OnSay", &vList);
}

void GameLogicComponent::OnDie()
{
	assert(!m_bDead);

	m_bDead = true;
	GetMessageManager()->DeleteMessagesToComponent(this); 
	GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_MUSIC, "audio/scary_hit.mp3", 50, TIMER_SYSTEM);

	Say("H4 Unit: `oSir, all life support systems have been knocked out. I think we should... sir?``", 1200, 4500);
	Say("`pYou have suffocated and/or frozen to death.`` `wClick `wBack`` to continue.``", 6000, 250000);

}

void GameLogicComponent::OnUpdate(VariantList *pVList)
{
	if (m_evolveTimer != 0 && m_evolveTimer < GetTick(TIMER_GAME))
	{
		if (!GetCity()->Evolve(1, CityData::EVOLVE_NORMAL))
		{
			m_evolveTimer = 0;
			GetMessageManager()->CallComponentFunction(GetGameLogic(), 0, "InitDemo");
		} else
		{
			m_evolveTimer = GetTick(TIMER_GAME) + 100;
		}
		m_pCityRender->UpdateCamera();
	}

	if (m_mode == MODE_GAME)
	{
		if (!m_bDead && m_cityData.GetLife() == 0)
		{
			OnDie();
		}
	}

	m_spawner.Update();
	
	GetProgressOfAttack();
	//LogMsg("Updating!");
}


void GameLogicComponent::Reset()
{
	m_spawner.Clear();

	m_bInPreviewMode = false;
	m_attackStart = m_attackLengthMS = 0;
	m_level = 0;
	m_bDead = false;
	GetMessageManager()->DeleteMessagesToComponent(this); 
	m_resources = 0;
	m_evolveTimer = 0;

	if (m_pCityRender)
	{
		GetParent()->RemoveComponentByAddress(m_pCityRender);
		m_pCityRender = NULL;
	}
}

void GameLogicComponent::InitUpgradeMenu()
{
	m_mapBackup = m_cityData;
	string msg;

	m_level++;
	
#ifdef _DEBUG
	//CHEAT level
	//if (m_level < 10) m_level = 10;
#endif
	SetDayText(m_level);

	//default rules for the level
	m_levelEvolveRules = CityData::EVOLVE_NORMAL;
	m_levelSpawnType = RockSpawner::TYPE_EASY;
	m_levelRockAttackTime = 30*1000;
	m_levelRockMinSpawnMS = 0.5*1000;
	m_levelRockMaxSpawnMS = 3*1000;
	m_levelRockColor = MAKE_RGBA(70,70,255,200);

	msg = "H4 Unit: `o Day "+toString(m_level)+". Check out these designs to use our `2+"+toString(m_resources)+"`` new resources on existing sockets.``";



	if (m_level > 7)
	{
		//better defaults
		m_levelSpawnType = RockSpawner::TYPE_HARD;
				m_levelRockAttackTime = (25+ ( (m_level-7)*3))*1000;
				m_levelRockMinSpawnMS = 0.5*1000;
				m_levelRockMaxSpawnMS = 3*1000;
				m_levelRockColor = GetBrightColor();
	}

	switch (m_level)
	{
		case 1:
			msg = "H4 Unit: `oWelcome sir! Choose an initial `wSpace City`` design and click OK.  You can `wadjust rotation`` on new pieces by clicking them.``";
			m_levelEvolveRules = CityData::EVOLVE_RICH;
	break;


		case 2:
			m_levelSpawnType = RockSpawner::TYPE_MEDIUM;
			m_levelRockAttackTime = 35*1000;
			m_levelRockColor = MAKE_RGBA(247,147,30,200); //orange

			break;

		case 3:
			m_levelSpawnType = RockSpawner::TYPE_EASY;
			m_levelRockAttackTime = 35*1000;
			m_levelRockMinSpawnMS = 0.5*1000;
			m_levelRockMaxSpawnMS = 1*1000;
			m_levelRockColor = MAKE_RGBA(264,72,97,200); //purple

			break;

		case 4:
			m_levelSpawnType = RockSpawner::TYPE_MEDIUM;
			m_levelRockAttackTime = 45*1000;
			m_levelRockMinSpawnMS = 0.5*1000;
			m_levelRockMaxSpawnMS = 3*1000;
			m_levelRockColor = MAKE_RGBA(172,172,172,255); //gray

			break;

	
		case 5:
			m_levelSpawnType = RockSpawner::TYPE_HARD;
			m_levelRockAttackTime = 25*1000;
			m_levelRockMinSpawnMS = 0.5*1000;
			m_levelRockMaxSpawnMS = 3*1000;
			m_levelRockColor = MAKE_RGBA(0,222,255,220); //cyan

			break;

		case 6:
			m_levelSpawnType = RockSpawner::TYPE_MEDIUM;
			m_levelRockAttackTime = 35*1000;
			m_levelRockMinSpawnMS = 0.3*1000;
			m_levelRockMaxSpawnMS = 2*1000;
			m_levelRockColor = MAKE_RGBA(255,255,255,255); //white

			break;

		case 7:
			m_levelSpawnType = RockSpawner::TYPE_HARD;
			m_levelRockAttackTime = 35*1000;
			m_levelRockMinSpawnMS = 0.5*1000;
			m_levelRockMaxSpawnMS = 3*1000;
			m_levelRockColor = MAKE_RGBA(254,255,82,220); //yellow

			break;

		case 10:
			m_levelSpawnType = RockSpawner::TYPE_HARD;
			m_levelRockAttackTime = 45*1000;
			m_levelRockMinSpawnMS = 0.5*1000;
			m_levelRockMaxSpawnMS = 2*1000;
		

			break;

		default:
			break;
	}

#ifdef _DEBUG
	//CHEAT level clear time
	//m_levelRockAttackTime = 5*1000;

#endif
	
	UpgradeMenuCreate(GetEntityRoot()->GetEntityByName("GUI"), msg);
}

void GameLogicComponent::OnInitUpgradeMenu(VariantList *pVList)
{
	InitUpgradeMenu();
}

void GameLogicComponent::HarvestResources(int &timeOut)
{
	
	vector<CityCell*> cells = m_cityData.GetCellsWithOverlayType(OVERLAY_GENERATOR);

	if (cells.empty())
	{
		//Say("H4 Unit: `oNo generators to harvest.``", timeOut, 2500);
		//timeOut += 2500;
	} else
	{
		Say("H4 Unit: `oCollecting resources from our `w"+toString(cells.size())+"`` generators(s).", timeOut, 3500);
		timeOut += 2500;

		for (int i=0; i < cells.size(); i++)
		{
			VariantList vList(GetRenderComponent()->WorldToScreen(GetRenderComponent()->GetCenterCellPosWorld(cells[i])));
			GetMessageManager()->CallComponentFunction(this, timeOut, "AddResourceFromPos", &vList);
			timeOut += 1000;
		}
	
		timeOut += 300;
	}

}

void GameLogicComponent::AddResourceFromPos(VariantList *pVList)
{
	AddResourcesFromPos(pVList->Get(0).GetVector2(), 1);
}

void GameLogicComponent::ShowProgressBar(VariantList *pVList)
{
	bool bShow = pVList->Get(0).GetUINT32() != 0;
	GUIShowProgressBar(bShow);
	
}

void GameLogicComponent::CollectBonusResources(int &timeOut)
{
	//give them some random stuff
	int bonusResources = RandomRange(2, 3);

	Say("H4 Unit: `oCollecting resources from debris.``", timeOut, 3500);

	timeOut += 2000;
	for (int i=0; i < bonusResources; i++)
	{
		VariantList vList(CL_Vec2f(RandomRange(0, GetScreenSizeXf()), RandomRange(200, GetScreenSizeYf())));
		GetMessageManager()->CallComponentFunction(this, timeOut, "AddResourceFromPos", &vList);
		timeOut += 1000;
	}

	//timeOut += 1000;
}

void GameLogicComponent::OnEndRockMode(VariantList *pVList)
{
	LogMsg("Rock mode ended");
	//CHEAT Ending rock attack
	m_attackStart = m_attackLengthMS = 0;
	int timeMS = 500;

	if (m_level == 10)
	{
		GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_MUSIC, "audio/win.mp3", 500, TIMER_SYSTEM);

		int msgInterval = 4000;
		Say("The station survived ten days.  A tribute to the winner:``", timeMS, timeMS+msgInterval); timeMS += msgInterval+500;
		Say("'A game made in 48 hours, evolution was the theme'``", timeMS, timeMS+msgInterval); timeMS += msgInterval+500;
		Say("'Pelted by rocks, they can't hear you scream'``", timeMS, timeMS+msgInterval); timeMS += msgInterval+500;
		Say("'They promised a vacation, a chance to be lazy'``", timeMS, timeMS+msgInterval); timeMS += msgInterval+500;
		Say("'What a crock, you ended up clicking like crazy'``", timeMS, timeMS+msgInterval); timeMS += msgInterval+500;
		Say("'When all was lost, you won the day with a bomb'``", timeMS, timeMS+msgInterval); timeMS += msgInterval+500;
		Say("'Thanks for playing, now go visit `ortsoft.com``!'``", timeMS, timeMS+msgInterval); timeMS += msgInterval+500;
		//Say("'Alone and scared, in space``", msgTimer, msgTimer+msgInterval); msgTimer += msgInterval+500;
		Say("'(returning to gameplay, it just gets harder)'``", timeMS, timeMS+msgInterval); timeMS += msgInterval+500;


	} else
	{
		GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_MUSIC, "audio/pensive.mp3", 500, TIMER_SYSTEM);

	}
	
	m_pCityRender->DestroyAllRocks(false);
	if (!m_bDead)
	{
		Say("H4 Unit: `oIt's over, sir.  We live, this time.``", timeMS, 2800);
		timeMS += 3500;

		CollectBonusResources(timeMS);
		HarvestResources(timeMS);
		GetMessageManager()->CallComponentFunction(GetGameLogic(), timeMS, "OnInitUpgradeMenu");
	}
}

void GameLogicComponent::FinalizeUpgrade()
{
	m_bInPreviewMode = false;

	m_resources = 0;
	GetMessageManager()->CallComponentFunction(this, 950, "UpdateResourceDisplay");

	m_spawner.Init(m_levelSpawnType, m_levelRockAttackTime, m_levelRockMinSpawnMS, m_levelRockMaxSpawnMS, m_levelRockColor);

	m_attackStart = GetTick(TIMER_GAME);
	m_attackLengthMS = m_levelRockAttackTime;
	//trigger next mode in a bit:
	
	//CHEAT Starting rock attack
	
	switch (m_level)
	{

	default:
		Say("H4 Unit: `oAsteroids in sector detected.``", 1000, 3000);
	}
	
	//play music

	int r = 0;
	if (r ==0)
	{
		GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_MUSIC, "audio/evil.mp3", 1000, TIMER_SYSTEM);
	} else if (r == 1)
	{
	//	GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_MUSIC, "audio/dirtybeat.mp3", 5000, TIMER_SYSTEM);
	}

	VariantList vList(uint32(1));
	GetMessageManager()->CallComponentFunction(this, 3000, "ShowProgressBar", &vList);

	vList = VariantList(uint32(0));

	GetMessageManager()->CallComponentFunction(this, 1000+m_levelRockAttackTime, "ShowProgressBar", &vList);

	GetMessageManager()->CallComponentFunction(this, m_levelRockAttackTime, "EndRockMode");
}

void GameLogicComponent::InitMap(VariantList *pVList)
{
	//CHEAT init map

	Reset();

	m_mode = MODE_GAME;

	m_pCityRender = (CityRenderComponent*) GetParent()->AddComponent(new CityRenderComponent);
	
	m_cityData.InitMap(20,12);
	m_resources = 5; //what you start with
	
#ifdef _DEBUG
	m_resources = 25;
#endif
	m_cityData.GenerateBase();
	//m_cityData.Evolve(10);

	//m_cityData.FillRandom();
	m_pCityRender->SetCityData(&m_cityData);
	
	GameMenuCreate(GetEntityRoot()->GetEntityByName("GUI"));
	
	//wire quit button directly to us
	GetEntityRoot()->GetEntityByName("Quit")->GetFunction("OnButtonSelected")->sig_function.connect(1, boost::bind(&GameLogicComponent::OnQuit, this, _1));
	m_pCityRender->UpdateCamera();
	
//	OnDie();
	InitUpgradeMenu();
}

void GameLogicComponent::OnQuit(VariantList *pVList)
{
	LogMsg("Clicked quit");
	GetMessageManager()->CallComponentFunction(GetGameLogic(), 200, "InitDemo");
	MainMenuCreate(GetEntityRoot()->GetEntityByName("GUI"));
}

void GameLogicComponent::InitDemo(VariantList *pVList)
{
	Reset();

	GetEntityRoot()->RemoveEntityByName("GameGUI", true);
	GetEntityRoot()->RemoveEntityByName("UpgradeMenu", true);

	m_pCityRender = (CityRenderComponent*) GetParent()->AddComponent(new CityRenderComponent);
	m_cityData.InitMap(20,12);
	m_cityData.GenerateBase();
	m_pCityRender->SetCityData(&m_cityData);
	m_evolveTimer = GetTick(TIMER_GAME) + 500;

	m_pCityRender->UpdateCamera(true);
	m_mode = MODE_DEMO;
}


void GameLogicComponent::OnClickCell( CityCell *pCell )
{
	if (pCell->IsClickable())
	{
		pCell->OnClicked();
		return;
	} else
	{
	}
}

CityData * GameLogicComponent::GetCity()
{
	return &m_cityData;
}

CL_Vec2f GetVectorDirFromOrientation(eCellOrientation orientation)
{
	switch(orientation)
	{
	case ORIENTATION_UP: return CL_Vec2f(0,-1);
	case ORIENTATION_DOWN: return CL_Vec2f(0,1);
	case ORIENTATION_LEFT: return CL_Vec2f(-1,0);
	case ORIENTATION_RIGHT: return CL_Vec2f(1,0);

	default:
		;
	}
	assert(!"Error");
	return CL_Vec2f(0,-1);
}

void GameLogicComponent::FireBulletFromCell( CityCell *pCell, int socket )
{
	CL_Vec2f vPos = m_pCityRender->GetCenterCellPosWorld(pCell);
	CL_Vec2f vDir = GetVectorDirFromOrientation((eCellOrientation) socket);

	//move it farther out
	vPos += (vDir * 30.0f);

	float rad = atan2(vDir.x, vDir.y);

	if (pCell->m_socket[socket] == SOCKET_WEAPON_DIAGONAL_RIGHT)
	{
		rad += -0.78f;
	}
	
	if (pCell->m_socket[socket] == SOCKET_WEAPON_DIAGONAL_LEFT)
	{
		rad += 0.78f;
	}

	vDir = CL_Vec2f(sin(rad), cos(rad));

	Bullet *pBullet = m_pCityRender->AddBullet(vPos, pCell, vDir);

}

void GameLogicComponent::PreviewUpgrade( int seed )
{
	m_cityData = m_mapBackup;

	m_cityData.ClearPreviewInfo();
	m_cityData.SetEvolveSeed(seed);
	
	
#ifdef _DEBUG
	m_cityData.Evolve(m_resources, CityData::EVOLVE_NORMAL);

#else
	m_cityData.Evolve(m_resources, m_levelEvolveRules);
#endif

	m_bInPreviewMode = true;
	m_pCityRender->UpdateCamera();
}

float GameLogicComponent::GetProgressOfAttack()
{
	if (m_attackLengthMS == 0) return 1.0f; //it's over

	float progress =  ((float) ((GetTick(TIMER_GAME)-m_attackStart))) / ((float) m_attackLengthMS);

	if (progress > 1) return 1.0f;
	
	return progress;
}