//  ***************************************************************
//  GameLogicComponent - Creation date: 8/25/2012 Tokyo Time
//  -------------------------------------------------------------
//  Robinson Technologies Copyright (C) 2012 - All Rights Reserved
//
//  ***************************************************************
//  Programmer(s):  Seth A. Robinson (seth@rtsoft.com)
//  ***************************************************************

#ifndef GameLogicComponent_h__
#define GameLogicComponent_h__

#include "Entity/Component.h"
#include "Entity/Entity.h"
#include "CityData.h"
#include "RockSpawner.h"

class CityRenderComponent;

//#include "Renderer/SurfaceAnim.h"

class GameLogicComponent: public EntityComponent
{
public:

	GameLogicComponent();
	virtual ~GameLogicComponent();

	virtual void OnAdd(Entity *pEnt);
	virtual void OnRemove();

	void InitMap(VariantList *pVList);
	void OnClickCell(CityCell *pCell);
	CityData * GetCity();
	void FireBulletFromCell(CityCell *pCell, int socket);
	bool IsDemoMode() {return m_mode == MODE_DEMO;}
	enum eMode
	{
		MODE_DEMO,
		MODE_GAME
	};

	void PreviewUpgrade(int seed);
	void FinalizeUpgrade();
	CityRenderComponent * GetRenderComponent(){return m_pCityRender;}
	void AddResourcesFromPos(CL_Vec2f vScreenPos, int resources);
	void OnUseNuke();
	float GetProgressOfAttack();
	int GetLevel(){return m_level;};
	bool InPreviewMode() {return m_bInPreviewMode;}

private:

	void OnRender(VariantList *pVList);
	void OnUpdate(VariantList *pVList);
	void InitDemo(VariantList *pVList);
	void Reset();
	void OnQuit(VariantList *pVList);
	void InitUpgradeMenu();
	void OnEndRockMode(VariantList *pVList);
	void OnUpdateResourceDisplay(VariantList *pVList);
	void Say(string msg, int delayMS, int timeToShowMS);
	void OnDie();
	void OnSay(VariantList *pVList);
	void OnInitUpgradeMenu(VariantList *pVList);
	void HarvestResources(int &timeOut);
	void CollectBonusResources(int &timeOut);
	void AddResourceFromPos(VariantList *pVList);
	void ShowProgressBar(VariantList *pVList);
	CL_Vec2f *m_pPos2d;
	CityData m_cityData;
	CityRenderComponent *m_pCityRender;
	eMode m_mode;
	int m_resources;
	
	
	bool m_bInPreviewMode;
	/*
	CL_Vec2f *m_pSize2d;
	CL_Vec2f *m_pScale2d;
	uint32 *m_pColor;
	uint32 *m_pColorMod;
	float *m_pAlpha;
	uint32 *m_pAlignment;
	float *m_pRotation; //in degrees
	*/
	uint32 m_evolveTimer;
	CityData m_mapBackup;
	RockSpawner m_spawner;
	int m_level;
	bool m_bDead;
	uint32 m_attackStart;
	uint32 m_attackLengthMS;

	int m_levelRockAttackTime;
	int m_levelRockMinSpawnMS;
	int m_levelRockMaxSpawnMS;
	RockSpawner::eType m_levelSpawnType;
	uint32 m_levelRockColor;

	CityData::eEvolveRules m_levelEvolveRules;
};

GameLogicComponent * GetGameLogic();
CityData * GetCity();

#endif // GameLogicComponent_h__