#include "PlatformPrecomp.h"
#include "MainMenu.h"
#include "Entity/EntityUtils.h"
#include "Entity/CustomInputComponent.h"
#include "Component/GameLogicComponent.h"
#include "HelpMenu.h"

void MainMenuOnSelect(VariantList *pVList) //0=vec2 point of click, 1=entity sent from
{
	Entity *pEntClicked = pVList->m_variant[1].GetEntity();
	LogMsg("Clicked %s entity at %s", pEntClicked->GetName().c_str(),pVList->m_variant[1].Print().c_str());
	Entity *pMenu = GetEntityRoot()->GetEntityByName("MainMenu");

	if (pEntClicked->GetName() == "Play")
	{
		//overlay the debug menu over this one
		pMenu->RemoveComponentByName("FocusInput");
		LogMsg("Clicked play");
	//	GetMessageManager()->CallComponentFunction(GetGameLogic(), 1, "InitMap");
		FadeOutAndKillEntity(pMenu, true, 500);
		HelpMenuCreate(pMenu->GetParent());
	}

	GetEntityRoot()->PrintTreeAsText(); //useful for debugging
}


Entity * MainMenuCreate(Entity *pParentEnt)
{
	GetMessageManager()->CallComponentFunction(GetGameLogic(), 0, "InitDemo");

	GetMessageManager()->SendGame(MESSAGE_TYPE_PLAY_MUSIC, "audio/dirtybeat.mp3", 50, TIMER_SYSTEM);


	Entity *pBG = CreateOverlayRectEntity(pParentEnt, CL_Vec2f(0,0), GetScreenSize(), MAKE_RGBA(0,0,0,150));
	pBG->SetName( "MainMenu");
	Entity *pTitle = CreateOverlayEntity(pBG, "Title", "game/title.rttex", GetScreenSizeXf()/2, 0);
	SetAlignmentEntity(pTitle, ALIGNMENT_UPPER_CENTER);
	AddFocusIfNeeded(pBG);
	
	Entity *pButtonEntity;
	
	eFont fontID = FONT_LARGE;
	float fontScale = EnforceMinimumFontLineToScreenRatio(fontID, 1.0f, 6.6f);
	float ySpacer = iPadMapY(10)+GetBaseApp()->GetFont(fontID)->GetLineHeight(fontScale);

	float x = GetScreenSizeXf()/2;
	float y = (GetScreenSizeYf()/2)-50;

	pButtonEntity = CreateTextButtonEntity(pBG, "Play",(x), (y), "Play", false); y += ySpacer;
	pButtonEntity->GetShared()->GetFunction("OnButtonSelected")->sig_function.connect(&MainMenuOnSelect);
	SetupTextEntity(pButtonEntity, fontID, fontScale);
	SetAlignmentEntity(pButtonEntity, ALIGNMENT_CENTER);
	//RemovePaddingEntity(pButtonEntity);
	
	//add version # for easy identification
	pButtonEntity = CreateTextLabelEntity(pBG, "version", GetScreenSizeXf()-iPhoneMapX(50), GetScreenSizeYf()-iPhoneMapY(20), GetApp()->GetVersionString());
	FadeInEntity(pButtonEntity);

	//for android, so the back key (or escape on windows/OSX) will quit out of the game
	EntityComponent *pComp = pBG->AddComponent(new CustomInputComponent);
	//tell the component which key has to be hit for it to be activated
	pComp->GetFunction("OnActivated")->sig_function.connect(1, boost::bind(&App::OnExitApp, GetApp(), _1));
	pComp->GetVar("keycode")->Set(uint32(VIRTUAL_KEY_BACK));

	//SlideScreen(pBG, true);
	FadeInEntity(pBG, true, 1000, 0);
	return pBG;
}
