#include "PlatformPrecomp.h"
#include "RockSpawner.h"
#include "Component/GameLogicComponent.h"
#include "Component/CityRenderComponent.h"

RockSpawner::RockSpawner()
{
}

RockSpawner::~RockSpawner()
{
}

void RockSpawner::SpawnRock()
{
	eSpawnType spawnType = (eSpawnType) m_rand.GetRandom();

	float size = 1.0f;
	float speed = RandomRangeFloat(1, 4);

	switch(spawnType)
	{
		
		case SPAWN_TINY:
		size = 0.20f;
		break;

		case SPAWN_SMALL:
			size = 0.35f;
			break;

		case SPAWN_MEDIUM:
			size = 0.5f;
			break;
		
		case SPAWN_BIG:
			size = 0.7f;
			break;

		case SPAWN_HUGE:
			size = 1.0f;
			break;


	default:;
	//	LogMsg("Spawn error");
	
	}

	Rock *pRock = GetGameLogic()->GetRenderComponent()->AddRock(size, speed);
	pRock->m_baseColor = m_color;

	if (GetGameLogic()->GetLevel() == 10)
	{
		pRock->m_baseColor = GetBrightColor();
	}
}

void RockSpawner::Update()
{
	if (m_timer < GetTick(TIMER_GAME))
	{
		return;
	}

	if (m_spawnTimer < GetTick(TIMER_GAME))
	{
		//time to spawn something
		SpawnRock();
		ScheduleNextEvent();
	}
}

void RockSpawner::ScheduleNextEvent()
{
	m_spawnTimer = GetTick(TIMER_GAME)+ RandomRange(m_rockMinMS, m_rockMaxMS);
}

void RockSpawner::Init( eType type, int lengthMS, int rockMinMS, int rockMaxMS, uint32 color )
{
	m_type = type;
	m_rand.Clear();
	m_color = color;
	m_timer = GetTick(TIMER_GAME)+lengthMS;
	m_rockMinMS = rockMinMS;
	m_rockMaxMS = rockMaxMS;
	
	m_spawnTimer = 0;
	
	switch(type)
	{
	case TYPE_EASY:
		m_rand.AddChoice(SPAWN_TINY, 100);
		m_rand.AddChoice(SPAWN_SMALL, 30);
		m_rand.AddChoice(SPAWN_MEDIUM, 10);
		m_rand.AddChoice(SPAWN_BIG, 0);
		m_rand.AddChoice(SPAWN_HUGE, 0);

		break;

	case TYPE_MEDIUM:
		m_rand.AddChoice(SPAWN_TINY, 100);
		m_rand.AddChoice(SPAWN_SMALL, 40);
		m_rand.AddChoice(SPAWN_MEDIUM, 20);
		m_rand.AddChoice(SPAWN_BIG, 5);
		m_rand.AddChoice(SPAWN_HUGE, 3);

		break;

	case TYPE_HARD:
		m_rand.AddChoice(SPAWN_TINY, 100);
		m_rand.AddChoice(SPAWN_SMALL, 100);
		m_rand.AddChoice(SPAWN_MEDIUM, 100);
		m_rand.AddChoice(SPAWN_BIG, 100);
		m_rand.AddChoice(SPAWN_HUGE, 100);

		break;
	}

	ScheduleNextEvent();
}

void RockSpawner::Clear()
{
	m_spawnTimer = 0;
	m_timer = 0;
}